import { Injector } from '@angular/core';
import { ClassTransformOptions } from 'class-transformer';
import { List } from 'immutable';
import { BehaviorSubject } from 'rxjs';
import { Observable } from 'rxjs';
import { Subject } from 'rxjs';
import { ProviderActionEnum } from '../enums/provider-action.enum';
import { IFactoryModel } from '../interfaces/factory-model';
import { IModel } from '../interfaces/model';
import { IPaginationMeta } from '../interfaces/pagination-meta';
import { IProvider } from '../interfaces/provider';
import { IProviderActionActionModel } from '../interfaces/provider-action-action-model';
import { IProviderActionOptions } from '../interfaces/provider-action-options';
import { IProviderOptions } from '../interfaces/provider-options';
export declare class Provider<TModel extends IModel = any> implements IProvider<TModel> {
    protected injector: Injector;
    protected factoryModel: IFactoryModel<TModel>;
    filter?: any;
    autoload?: boolean;
    delay: number;
    name: string;
    items$: BehaviorSubject<List<TModel>>;
    action$: Subject<IProviderActionActionModel>;
    create$: Subject<TModel>;
    append$: Subject<TModel>;
    update$: Subject<TModel>;
    patch$: Subject<TModel>;
    delete$: Subject<TModel>;
    load$: Subject<TModel>;
    loadAll$: Subject<TModel[]>;
    actionIsActive$: BehaviorSubject<boolean>;
    createIsActive$: BehaviorSubject<boolean>;
    updateIsActive$: BehaviorSubject<boolean>;
    deleteIsActive$: BehaviorSubject<boolean>;
    loadIsActive$: BehaviorSubject<boolean>;
    loadAllIsActive$: BehaviorSubject<boolean>;
    paginationMeta$: BehaviorSubject<IPaginationMeta>;
    protected prevOptions?: IProviderActionOptions;
    protected destroy$: Subject<boolean>;
    constructor(injector: Injector, factoryModel: IFactoryModel<TModel>);
    instanceofFactoryModel(data: any): boolean;
    instanceofNestedFactoryModel(data: any): boolean;
    updateNestedFactoryModel(data: any): void;
    deleteNestedFactoryModel(data: any): void;
    plainToClass(data: any, action: ProviderActionEnum, classTransformOptions?: ClassTransformOptions): TModel;
    classToPlain(model: TModel, action: ProviderActionEnum, classTransformOptions?: ClassTransformOptions): Object;
    classToClass(model: TModel, classTransformOptions?: ClassTransformOptions): TModel;
    action<TProviderActionOptions extends IProviderActionOptions>(key: string, data?: any, options?: TProviderActionOptions): Observable<any>;
    save<TProviderActionOptions extends IProviderActionOptions>(model: TModel, options?: TProviderActionOptions): Observable<TModel>;
    create<TProviderActionOptions extends IProviderActionOptions>(model: TModel, options?: TProviderActionOptions): Observable<TModel>;
    append<TProviderActionOptions extends IProviderActionOptions>(model: TModel, options?: TProviderActionOptions): Observable<TModel>;
    update<TProviderActionOptions extends IProviderActionOptions>(key: number | string, model: TModel, options?: TProviderActionOptions): Observable<TModel>;
    patch<TProviderActionOptions extends IProviderActionOptions>(key: number | string, model: TModel, options?: TProviderActionOptions): Observable<TModel>;
    delete<TProviderActionOptions extends IProviderActionOptions>(key: number | string, options?: TProviderActionOptions): Observable<TModel>;
    load<TProviderActionOptions extends IProviderActionOptions>(key: number | string, options?: TProviderActionOptions): Observable<TModel>;
    loadAll<TProviderActionOptions extends IProviderActionOptions>(filter?: any, options?: TProviderActionOptions): Observable<TModel[]>;
    calcPaginationMetaByOptions(options: IProviderOptions<TModel>): IPaginationMeta;
    calcPaginationMeta(newPaginationMeta: IPaginationMeta): IPaginationMeta;
    reloadAll(): void;
    checkFilterAndOptions(): void;
    setOptions(options: IProviderOptions<TModel>): void;
    getOptions(): IProviderOptions<TModel>;
    reconfigItems(items?: List<TModel>): void;
}
