import { Injector } from '@angular/core';
import { List } from 'immutable';
import { Observable } from 'rxjs';
import { IFactoryModel } from '../interfaces/factory-model';
import { IHttpClient } from '../interfaces/http-client';
import { IModel } from '../interfaces/model';
import { IProviderOptions } from '../interfaces/provider-options';
import { IRestOptions } from '../interfaces/rest-options';
import { IRestProviderActionHandlers } from '../interfaces/rest-provider-action-handlers';
import { IRestProviderActionOptions } from '../interfaces/rest-provider-action-options';
import { IRestProviderOptions } from '../interfaces/rest-provider-options';
import { FakeHttpClient } from '../utils/fake-http-client';
import { Provider } from './provider';
export declare class RestProvider<TModel extends IModel> extends Provider<TModel> {
    protected injector: Injector;
    protected factoryModel: IFactoryModel<TModel>;
    protected options: IProviderOptions<TModel>;
    pluralName: string;
    name: string;
    apiUrl: string;
    httpClient: IHttpClient;
    fakeHttpClient: FakeHttpClient;
    restOptions: IRestOptions;
    providerActionHandlers: IRestProviderActionHandlers;
    constructor(injector: Injector, factoryModel: IFactoryModel<TModel>, options: IProviderOptions<TModel>);
    initFakeHttpClient(): void;
    init(): void;
    getOptions(): IRestProviderOptions<TModel>;
    setOptions(options?: IRestProviderOptions<TModel>): void;
    action<TProviderActionOptions extends IRestProviderActionOptions>(key: string, data?: any, options?: TProviderActionOptions): Observable<TModel>;
    create<TProviderActionOptions extends IRestProviderActionOptions>(model: TModel, options?: TProviderActionOptions): Observable<TModel>;
    private _update;
    update<TProviderActionOptions extends IRestProviderActionOptions>(key: number | string, model: TModel, options?: TProviderActionOptions): Observable<TModel>;
    patch<TProviderActionOptions extends IRestProviderActionOptions>(key: number | string, model: TModel, options?: TProviderActionOptions): Observable<TModel>;
    load<TProviderActionOptions extends IRestProviderActionOptions>(key: number | string, options?: TProviderActionOptions): Observable<TModel>;
    delete<TProviderActionOptions extends IRestProviderActionOptions>(key: number | string, options?: TProviderActionOptions): Observable<TModel>;
    reloadAll(): void;
    loadAll<TProviderActionOptions extends IRestProviderActionOptions>(filter?: any, options?: TProviderActionOptions): Observable<any>;
    private _loadAll;
    reconfigItems(items?: List<TModel>): void;
}
