import { Injector, OnDestroy } from '@angular/core';
import { BehaviorSubject } from 'rxjs';
import { Observable } from 'rxjs';
import { IFactoryModel } from '../interfaces/factory-model';
import { IMockProviderOptions } from '../interfaces/mock-provider-options';
import { IModel } from '../interfaces/model';
import { IProvider } from '../interfaces/provider';
import { IProviderOptions } from '../interfaces/provider-options';
import { IRestProviderActionOptions } from '../interfaces/rest-provider-action-options';
import { IRestProviderOptions } from '../interfaces/rest-provider-options';
export declare class Repository<TModel extends IModel = any> implements OnDestroy {
    protected injector: Injector;
    protected factoryModel?: IFactoryModel<TModel>;
    protected factoryMockProvider?: {
        new (injector: Injector, factoryModel: IFactoryModel<TModel>, options: IMockProviderOptions<IModel>): IProvider<IModel>;
    };
    protected factoryRestProvider?: {
        new (injector: Injector, factoryModel: IFactoryModel<TModel>, options: IRestProviderOptions<IModel>): IProvider<IModel>;
    };
    mockProvider?: IProvider<TModel>;
    restProvider?: IProvider<TModel>;
    protected root: Repository<IModel>;
    protected mockProviders: IProvider<TModel>[];
    protected restProviders: IProvider<TModel>[];
    readonly provider: IProvider<TModel>;
    readonly paginationMeta$: BehaviorSubject<any>;
    readonly providers: IProvider<TModel>[];
    readonly items: TModel[];
    readonly items$: Observable<TModel[]>;
    private _mockProviderIsActive;
    private _restProviderIsActive;
    private destroyed$;
    constructor(injector: Injector, factoryModel?: IFactoryModel<TModel>, factoryMockProvider?: {
        new (injector: Injector, factoryModel: IFactoryModel<TModel>, options: IMockProviderOptions<IModel>): IProvider<IModel>;
    }, factoryRestProvider?: {
        new (injector: Injector, factoryModel: IFactoryModel<TModel>, options: IRestProviderOptions<IModel>): IProvider<IModel>;
    });
    ngOnDestroy(): void;
    fork<TNewModel extends IModel = TModel>(newFactoryModel?: IFactoryModel<TNewModel>, newFactoryMockProvider?: {
        new (injector: Injector, factoryModel: IFactoryModel<TNewModel>, options: IMockProviderOptions<IModel>): IProvider<IModel>;
    }, newFactoryRestProvider?: {
        new (injector: Injector, factoryModel: IFactoryModel<TNewModel>, options: IRestProviderOptions<IModel>): IProvider<IModel>;
    }): any;
    useMock<TMockProviderOptions extends IMockProviderOptions<IModel>>(options?: TMockProviderOptions): void;
    useRest<TMockProviderOptions extends IRestProviderOptions<IModel>>(options?: IRestProviderOptions<TModel>): void;
    subscribeToProvider(provider: IProvider<TModel>): void;
    setOptions(options: IProviderOptions<TModel>): void;
    action<TProviderActionOptions extends IRestProviderActionOptions>(key: string, data?: any, options?: TProviderActionOptions): Observable<any>;
    save<TProviderActionOptions extends IRestProviderActionOptions>(model: TModel, options?: TProviderActionOptions): Observable<TModel>;
    create<TProviderActionOptions extends IRestProviderActionOptions>(model: TModel, options?: TProviderActionOptions): Observable<TModel>;
    update<TProviderActionOptions extends IRestProviderActionOptions>(key: number | string, model: TModel, options?: TProviderActionOptions): Observable<TModel>;
    patch<TProviderActionOptions extends IRestProviderActionOptions>(key: number | string, model: TModel, options?: TProviderActionOptions): Observable<TModel>;
    load<TProviderActionOptions extends IRestProviderActionOptions>(key: number | string, options?: TProviderActionOptions): Observable<TModel>;
    delete<TProviderActionOptions extends IRestProviderActionOptions>(key: number | string, options?: TProviderActionOptions): Observable<TModel>;
    reloadAll(): void;
    loadAll<TProviderActionOptions extends IRestProviderActionOptions>(filter?: any, options?: TProviderActionOptions): Observable<TModel[]>;
    clone(model: TModel): any;
}
